import 'package:eyflutter_tik_video/models/collect_model.dart';
import 'package:eyflutter_tik_video/models/portrait_model.dart';
import 'package:eyflutter_tik_video/models/share_model.dart';
import 'package:eyflutter_tik_video/models/video_loading_model.dart';
import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/models/video_player_icon_model.dart';
import 'package:eyflutter_tik_video/tiklist/tik_video_gesture.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class TikVideoView extends StatelessWidget {
  /// 播放组件
  final FijkPlayer player;

  /// 点赞回调
  final Function()? favoriteCall;

  /// 单击回调
  final Function()? singleTapCall;

  /// 构建完成回调
  final Function() buildCall;

  /// 播放进度
  final VideoPlayProgressModel progressModel;

  /// 播放状态
  final VideoPlayerIconModel playerIconModel;

  /// 加载
  final VideoLoadingModel loadingModel;

  /// 收藏
  final CollectModel collectModel;

  /// 分享
  final ShareModel shareModel;

  /// 视频展现模式
  final FijkFit fit;

  /// 进度条颜色
  final Color progressColor;

  /// 进度条背景
  final Color progressBackgroundColor;

  final Function(PortraitModel model)? portraitCall;

  final Function(CollectModel model)? collectCall;

  final Function(ShareModel model)? shareCall;

  const TikVideoView({
    Key? key,
    required this.player,
    this.favoriteCall,
    this.singleTapCall,
    required this.buildCall,
    required this.progressModel,
    required this.playerIconModel,
    required this.loadingModel,
    this.fit = FijkFit.contain,
    required this.progressColor,
    required this.progressBackgroundColor,
    required this.collectModel,
    required this.shareModel,
    this.portraitCall,
    this.collectCall,
    this.shareCall,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        TikVideoGesture(
          favoriteCall: favoriteCall,
          singleTapCall: singleTapCall,
          playIconTap: playIconTap,
          buildCall: buildCall,
          progressModel: progressModel,
          playerIconModel: playerIconModel,
          loadingModel: loadingModel,
          progressColor: progressColor,
          progressBackgroundColor: progressBackgroundColor,
          collectModel: collectModel,
          shareModel: shareModel,
          portraitCall: portraitCall,
          collectCall: collectCall,
          shareCall: shareCall,
          child: Stack(
            children: <Widget>[
              Container(
                height: double.infinity,
                width: double.infinity,
                alignment: Alignment.center,
                color: const Color(0xff020210),
                child: buildVideo(),
              ),
            ],
          ),
        ),
      ],
    );
  }

  Widget buildVideo() {
    return Center(
      child: FijkView(
        width: double.infinity,
        height: double.infinity,
        fit: fit,
        player: player,
        color: Colors.transparent,
        panelBuilder: (_, __, ___, ____, _____) => const SizedBox(),
      ),
    );
  }

  void playIconTap(VideoPlayerIconModel model) {
    if (player.state == FijkState.started) {
      player.pause();
      model.isPlaying = false;
    } else {
      if (player.state == FijkState.prepared ||
          player.state == FijkState.paused ||
          player.state == FijkState.completed) {
        player.start();
        model.isPlaying = true;
      } else {
        model.isPlaying = false;
      }
    }
    model.notifierLazy();
  }
}
