import 'package:eyflutter_tik_video/beans/video_tik_tab.dart';
import 'package:eyflutter_tik_video/models/collect_model.dart';
import 'package:eyflutter_tik_video/models/portrait_model.dart';
import 'package:eyflutter_tik_video/models/share_model.dart';
import 'package:eyflutter_tik_video/models/video_tik_list_model.dart';
import 'package:eyflutter_tik_video/tiklist/video_tik_list_view.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class VideoListTabView extends StatefulWidget {
  /// tab标签
  final List<VideoTikTab> tabs;

  /// 创建PageView页面回调
  final Function(VideoTikTab tikTab, VideoTikListModel tikModel) pageCreated;

  /// tab指示器颜色
  final Color? tabIndicatorColor;

  /// tab指示器高度
  final double? tabIndicatorHeight;

  /// tab文本颜色
  final Color? tabTextColor;

  /// tab选中文本颜色
  final Color? tabSelectedTextColor;

  /// 触发加载下一页
  final Function(VideoTikTab tikTab) nextPage;

  /// 视频填充方式
  final FijkFit fit;

  /// 进度条颜色
  final Color progressColor;

  /// 进度条背景
  final Color progressBackgroundColor;

  final Function(PortraitModel model)? portraitCall;

  final Function(CollectModel model)? collectCall;

  final Function(ShareModel model)? shareCall;

  const VideoListTabView({
    Key? key,
    required this.tabs,
    required this.pageCreated,
    this.tabIndicatorColor,
    this.tabIndicatorHeight,
    this.tabTextColor,
    this.tabSelectedTextColor,
    required this.nextPage,
    this.fit = FijkFit.contain,
    this.progressColor = const Color(0xffFEFDFF),
    this.progressBackgroundColor = const Color(0xff7F7776),
    this.portraitCall,
    this.collectCall,
    this.shareCall,
  }) : super(key: key);

  @override
  _VideoListTabViewState createState() => _VideoListTabViewState();
}

class _VideoListTabViewState extends LifecycleState<VideoListTabView> with SingleTickerProviderStateMixin {
  @override
  String get viewName => "videoListTabView";

  TabController? tabController;
  int currentTabIndex = 0;
  List<Tab> tabItems = [];
  List<VideoTikListModel> listModels = [];

  @override
  void initState() {
    super.initState();
    tabController = TabController(length: widget.tabs.length, vsync: this);
    tabController?.addListener(() {
      currentTabIndex = tabController?.index ?? 0;
      setFrontPageTag();
    });
    for (var item in widget.tabs) {
      tabItems.add(Tab(text: item.tabName));
    }
  }

  void setFrontPageTag() {
    var length = listModels.length;
    for (int i = 0; i < length; i++) {
      var element = listModels[i];
      if (i == currentTabIndex) {
        element.isFrontPage = true;
      } else {
        element.isFrontPage = false;
      }
      element.changeCurrentPlayState();
    }
  }

  void changeBackgroundVideoState() {
    var length = listModels.length;
    for (int i = 0; i < length; i++) {
      var element = listModels[i];
      element.isFrontPage = false;
      element.changeCurrentPlayState();
    }
  }

  @override
  void onPause(String routeName, Map<String, dynamic> arguments) {
    changeBackgroundVideoState();
  }

  @override
  void onResume(String routeName, Map<String, dynamic> arguments) {
    setFrontPageTag();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return SafeArea(
      child: buildRootBody(),
      top: true,
    );
  }

  Widget buildRootBody() {
    return Scaffold(
      body: Stack(
        children: <Widget>[
          Positioned(
            left: 0,
            right: 0,
            top: 0,
            bottom: 0,
            child: Container(
              color: Colors.black,
            ),
          ),
          Positioned(
            left: 0,
            right: 0,
            top: 0,
            bottom: 0,
            child: buildTableViewWidget(),
          ),
          buildTabBarWidget(),
        ],
      ),
    );
  }

  buildTabBarWidget() {
    return Container(
      alignment: Alignment.topCenter,
      width: double.infinity,
      child: TabBar(
        controller: tabController,
        tabs: tabItems,
        indicatorColor: widget.tabIndicatorColor,
        indicatorWeight: widget.tabIndicatorHeight ?? 2.0,
        isScrollable: true,
        indicatorSize: TabBarIndicatorSize.tab,
        labelColor: widget.tabSelectedTextColor,
        unselectedLabelColor: widget.tabTextColor,
      ),
    );
  }

  buildTableViewWidget() {
    return TabBarView(
      controller: tabController,
      children: widget.tabs.map((value) => buildTableViewItemWidget(value)).toList(),
    );
  }

  Widget buildTableViewItemWidget(VideoTikTab tab) {
    return VideoTikListView(
      key: Key(tab.tag),
      buildCall: (VideoTikListModel model) {
        widget.pageCreated.call(tab, model);
        model.tikTab = tab;
        model.nextPage = widget.nextPage;
        listModels.add(model);
      },
      fit: widget.fit,
      progressColor: widget.progressColor,
      progressBackgroundColor: widget.progressBackgroundColor,
      portraitCall: widget.portraitCall,
      collectCall: widget.collectCall,
      shareCall: widget.shareCall,
    );
  }
}
